//-----------------------------------------------------------------------------
// ETH_LO_OSI3_ICMP
// Version 3.0
// 17.12.2011 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Lo-Level Funktionen
// mit dem Ethernet-IC : ENC28J60
//
// Funktionen : OSI-Schicht-3 implementation
//              ICMP-Dataframe (ETH-Typfield = 0x0800, IP-Typ=0x01)
//              (wird von OSI-Schicht-3 (IP) aufgerufen)
//
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_EXT_MODULE/UB_ETH_LOLEVEL/"
// liegen und mit #include <UB_ETH_LOLEVEL/ETH_LO_OSI3_ICMP.h>
// eingebunden werden
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//
// ICMP-Packet (Komplett)
// ----------------------
// ICMP-Laenge      = xx Bytes
// Start im Array = ab Adresse 34 (0 bis 13 = ETH-Header, 14 bis 33 = IP-Header)
// Packetlaenge   = xx Bytes (ETH=14 + IP=20 + ICMP=xx)
//
// ICMP-Packet (Header)
// ----------------------
// Laenge         = 8 Bytes
// Adr 34         = Typ (1 Byte) [Echo Request, Echo Reply usw]
// Adr 35         = Code (1 byte) [0x00]
// Adr 36 - 37    = ICMP-Chksum (2 Bytes)
// Adr 38 - 41    = Identifikation (4 Bytes)
//
// ICMP-Packet (Daten)
// ----------------------
// im Anschluss an den ICMP-Header folgen
// die ICMP-Daten (mindestens 18Byte, bei Echo = 32 Bytes)
//-----------------------------------------------------------------------------

#ifndef UB_ETH_ICMP    // Wichtig !!
#define UB_ETH_ICMP    // damit wird das File nur einmal eingebunden

//----------------------------------------------------------------------------
// Includes
//----------------------------------------------------------------------------
// keine



//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//-----------------------------------------------------------------------------
// siehe ETH_LO_GLOBAL.h

//-----------------------------------------------------------------------------
// Funktionen die von extern benutzt werden koennen
//-----------------------------------------------------------------------------
bool ICMP_executeICMPPacket(void);

//-----------------------------------------------------------------------------
// interne Funktionen
//-----------------------------------------------------------------------------
ETH_ICMP_TYP_t ICMP_checkICMPTyp(void);
void ICMP_readICMPData(void);
void ICMP_makeICMPReply(void);
void ICMP_makeICMPChecksum(void);


//-----------------------------------------------------------------------------
// interne Variabeln
//-----------------------------------------------------------------------------
unsigned char eth_icmp_identify[4];  // ICMP-Identify

//-----------------------------------------------------------------------------
// interne Defines
//-----------------------------------------------------------------------------
#define  ICMP_POS_FRAME  34 // start vom ICMP-Frame
#define  ICMP_POS_TYP    34 // position vom ICMP-Protokoll-Typ
#define  ICMP_POS_CRC_HI 36 // position der CRC (Hi-Byte)
#define  ICMP_POS_CRC_LO 37 // position der CRC (Lo-Byte)
#define  ICMP_POS_ID     38 // position vom ICMP-Identify

//-----------------------------------------------------------------------------
bool ICMP_executeICMPPacket(void)
{
  // barbeiten von ICMP-Packeten
  // MAC und IP wurde schon vom IP-Protokoll geprueft
  //
  // rueckgabe von true, wenn Packet bearbeitet wurde
  // setzt u.U. "TX_ETH_Data.tx_typ"

  bool ret_wert=false;
  ETH_ICMP_TYP_t icmp_typ=ETH_ICMP_UNKNWON;

  // pruefen auf ICMP-Protokoll-Typ
  icmp_typ=ICMP_checkICMPTyp();  
  if(icmp_typ==ETH_ICMP_REQUEST) {
    // wenn es eine ICMP-Anforderung war
    // alle notwendigen Daten retten
    ICMP_readICMPData();
    // und die ICMP-Antwort erstellen
    ICMP_makeICMPReply();
    // ICMP-Checksumme hinzufuegen
    ICMP_makeICMPChecksum();
    // damit eine Antwort gesendet wird
    TX_ETH_Data.tx_typ=ETH_TX_ICMP_REPLY;
    // TRUE zurueckgeben
    ret_wert=true;
  }

  return(ret_wert);
}

//-----------------------------------------------------------------------------
ETH_ICMP_TYP_t ICMP_checkICMPTyp(void)
{
  // prueft welcher ICMP-Protokoll-Typ es ist
  // Echo-Anforderung (request) = 0x08
  // Echo-Antwort (reply) = 0x00

  ETH_ICMP_TYP_t ret_wert=ETH_ICMP_UNKNWON;

  if(RX_ETH_Data.raw_data[ICMP_POS_TYP]==0x08) ret_wert=ETH_ICMP_REQUEST;
  if(RX_ETH_Data.raw_data[ICMP_POS_TYP]==0x00) ret_wert=ETH_ICMP_REPLY;

  return(ret_wert);
}

//-----------------------------------------------------------------------------
void ICMP_readICMPData(void) {
  // alle notwendigen Daten
  // vom ICMP-Protokoll speichern

  // Quell-MAC und IP-Adresse wurden schon gespeichert

  // ICMP-Identify speichern
  eth_icmp_identify[0]=RX_ETH_Data.raw_data[ICMP_POS_ID+0];
  eth_icmp_identify[1]=RX_ETH_Data.raw_data[ICMP_POS_ID+1];
  eth_icmp_identify[2]=RX_ETH_Data.raw_data[ICMP_POS_ID+2];
  eth_icmp_identify[3]=RX_ETH_Data.raw_data[ICMP_POS_ID+3];
}

//-----------------------------------------------------------------------------
void ICMP_makeICMPReply(void) {
  // ICMP-Reply-Packet erstellen

  unsigned char lo,hi;

  //---------------------------
  // ETH-Header
  //---------------------------
  // Ziel MAC-Adresse
  RX_ETH_Data.raw_data[0]=HOST_ETH_Data.host_mac[0];
  RX_ETH_Data.raw_data[1]=HOST_ETH_Data.host_mac[1];
  RX_ETH_Data.raw_data[2]=HOST_ETH_Data.host_mac[2];
  RX_ETH_Data.raw_data[3]=HOST_ETH_Data.host_mac[3];
  RX_ETH_Data.raw_data[4]=HOST_ETH_Data.host_mac[4];
  RX_ETH_Data.raw_data[5]=HOST_ETH_Data.host_mac[5];

  // Quell MAC-Adresse
  RX_ETH_Data.raw_data[6]=ETH_Setting.own_mac[0];
  RX_ETH_Data.raw_data[7]=ETH_Setting.own_mac[1];
  RX_ETH_Data.raw_data[8]=ETH_Setting.own_mac[2];
  RX_ETH_Data.raw_data[9]=ETH_Setting.own_mac[3];
  RX_ETH_Data.raw_data[10]=ETH_Setting.own_mac[4];
  RX_ETH_Data.raw_data[11]=ETH_Setting.own_mac[5];

  // Typfield (IP-Packet = 0x0800)
  RX_ETH_Data.raw_data[12]=0x08;
  RX_ETH_Data.raw_data[13]=0x00;

  //---------------------------
  // IP-Header
  //---------------------------

  // IP-Version und Header Laenge (IP=4, Laenge=5)
  RX_ETH_Data.raw_data[14]=0x45;

  // Typ of Service
  RX_ETH_Data.raw_data[15]=0x00;

  // Laenge des IP-Frames (IP-Header + IP-Data)  
  lo=(RX_ETH_Data.ip_frame_len&0x00FF);
  hi=(RX_ETH_Data.ip_frame_len>>8);
  RX_ETH_Data.raw_data[16]=hi;
  RX_ETH_Data.raw_data[17]=lo;

  // eindeutige ID-Nummer vom IP-Protokoll
  TX_ETH_Data.ip_id_nr++;
  lo=(TX_ETH_Data.ip_id_nr&0x00FF);
  hi=(TX_ETH_Data.ip_id_nr>>8);
  RX_ETH_Data.raw_data[18]=hi;
  RX_ETH_Data.raw_data[19]=lo;

  // Offset (bei Protokoll Fragment)
  RX_ETH_Data.raw_data[20]=0x00;
  RX_ETH_Data.raw_data[21]=0x00;

  // Time To Live
  RX_ETH_Data.raw_data[22]=0x80;

  // IP-Protokoll-Typ (ICMP = 0x01)
  RX_ETH_Data.raw_data[23]=0x01;

  // 24+25 = IP-Header Checksumme
  // wird spter von IP.h hinzugefuegt
  RX_ETH_Data.raw_data[24]=0x00;
  RX_ETH_Data.raw_data[25]=0x00;  

  // Quell IP-Adresse
  RX_ETH_Data.raw_data[26]=ETH_Setting.own_ip[0];
  RX_ETH_Data.raw_data[27]=ETH_Setting.own_ip[1];
  RX_ETH_Data.raw_data[28]=ETH_Setting.own_ip[2];
  RX_ETH_Data.raw_data[29]=ETH_Setting.own_ip[3];

  // Ziel IP-Adresse
  RX_ETH_Data.raw_data[30]=HOST_ETH_Data.host_ip[0];
  RX_ETH_Data.raw_data[31]=HOST_ETH_Data.host_ip[1];
  RX_ETH_Data.raw_data[32]=HOST_ETH_Data.host_ip[2];
  RX_ETH_Data.raw_data[33]=HOST_ETH_Data.host_ip[3];

  //---------------------------
  // ICMP-Header
  //---------------------------
  // ICMP-Protokoll-Typ (ICMP-Reply = 0x00)
  RX_ETH_Data.raw_data[34]=0x00;

  // Code-Nr
  RX_ETH_Data.raw_data[35]=0x00;

  // 36+37 = ICMP Checksumme
  // wird spter von ICMP.h hinzugefuegt
  RX_ETH_Data.raw_data[36]=0x00;
  RX_ETH_Data.raw_data[37]=0x00;

  // ICMP-Identify zuruecksenden
  RX_ETH_Data.raw_data[38]=eth_icmp_identify[0];
  RX_ETH_Data.raw_data[39]=eth_icmp_identify[1];
  RX_ETH_Data.raw_data[40]=eth_icmp_identify[2];
  RX_ETH_Data.raw_data[41]=eth_icmp_identify[3];

  //---------------------------
  // ICMP-Daten
  //---------------------------

  // Das Daten-Feld Adr 42 bis xx 
  // ist schon vom Sender gefuellt
  // und wird als ECHO zurueckgegeben

  // laenge festlegen
  // von der Framelaenge 4 Byte abziehen (wegen der ETH-CRC)
  TX_ETH_Data.eth_tx_len=RX_ETH_Data.eth_frame_len-4;

  //-----------------------------------------
  // die ETH-CRC wird vom EN28J60 angehaengt
  //-----------------------------------------
}


//-----------------------------------------------------------------------------
void ICMP_makeICMPChecksum(void) {
  // fuegt die ICMP-Checksumme in den Frame ein

  unsigned int icmp_crc=0;
  unsigned char lo,hi;

  // crc erstellen ueber den kompletten ICMP-Frame
  icmp_crc=ETH_makeChecksum(ICMP_POS_FRAME,RX_ETH_Data.ip_data_len);

  lo=(icmp_crc&0x00FF);
  hi=(icmp_crc>>8);

  // crc im Frame speichern
  RX_ETH_Data.raw_data[ICMP_POS_CRC_HI]=hi;
  RX_ETH_Data.raw_data[ICMP_POS_CRC_LO]=lo;
}

#endif  // Ende von #define UB_ETH_ICMP


