//-----------------------------------------------------------------------------
// ETH_LO_OSI4_UDP
// Version 3.0
// 17.12.2011 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Lo-Level Funktionen
// mit dem Ethernet-IC : ENC28J60
//
// Funktionen : OSI-Schicht-4 implementation
//              UDP-Dataframe (ETH-Typfield = 0x0800, IP-Typ=0x11)
//              (wird von OSI-Schicht-3 (IP) aufgerufen)
//
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_EXT_MODULE/UB_ETH_LOLEVEL/"
// liegen und mit #include <UB_ETH_LOLEVEL/ETH_LO_OSI4_UDP.h>
// eingebunden werden
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//
// UDP-Packet (Komplett)
// ----------------------
// UDP-Laenge     = xx Bytes
// Start im Array = ab Adresse 34 (0 bis 13 = ETH-Header, 14 bis 33 = IP-Header)
// Packetlaenge   = xx Bytes (ETH=14 + IP=20 + UDP=xx)
//
// UDP-Packet (Header)
// ----------------------
// Laenge         = 8 Bytes
// Adr 34 - 35    = Quell-Port (2 Byte)
// Adr 36 - 37    = Ziel-Port (2 byte)
// Adr 38 - 39    = Laenge (2 Bytes) [UDP-Header+UDP-Daten]
// Adr 40 - 41    = UDP-Chksum (4 Bytes)
//
// UDP-Packet (Daten)
// ----------------------
// im Anschluss an den UDP-Header folgen
// die UDP-Daten
//-----------------------------------------------------------------------------



#ifndef UB_ETH_UDP    // Wichtig !!
#define UB_ETH_UDP    // damit wird das File nur einmal eingebunden

//----------------------------------------------------------------------------
// Includes
//----------------------------------------------------------------------------
// keine


//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//-----------------------------------------------------------------------------
// siehe ETH_LO_GLOBAL.h


//-----------------------------------------------------------------------------
// Funktionen die von extern benutzt werden koennen
//-----------------------------------------------------------------------------
bool UDP_executeUDPPacket(void);
bool UDP_receivePacket(char *ptr);
void UDP_sendPacket(const char *ptr);

//-----------------------------------------------------------------------------
// interne Funktionen
//-----------------------------------------------------------------------------
void UDP_readUDPData(void);
void UDP_makeUDPPacket(void);
void UDP_makeUDPChecksum(void);



//-----------------------------------------------------------------------------
// interne Defines
//-----------------------------------------------------------------------------
#define  UDP_HEADER_LEN         8  // Headerlaenge (UDP_Header)
#define  UDP_FRAME_EIU_LEN      42 // Headerlaenge (ETH+IP+UDP_Header)
#define  UDP_FRAME_IU_LEN       28 // Headerlaenge (IP+UDP_Header)
#define  UDP_DATA_LEN  ETH_MAX_UDP_SIZE-UDP_FRAME_EIU_LEN
#define  UDP_POS_Q_PORT         34 // position vom Quellport (2 Bytes)
#define  UDP_POS_Z_PORT         36 // position vom Zielport (2 Bytes)
#define  UDP_POS_LEN            38 // position der Laenge (2 Bytes)
#define  UDP_POS_CRC            40 // position der CRC (2 Bytes)
#define  UDP_POS_DATA           42 // position der UDP-Daten


//-----------------------------------------------------------------------------
// interne Variabeln
//-----------------------------------------------------------------------------
// keine


//-----------------------------------------------------------------------------
bool UDP_executeUDPPacket(void)
{
  // barbeiten von UDP-Packeten
  // MAC und IP wurde schon vom IP-Protokoll geprueft
  //
  // rueckgabe von true, wenn Packet bearbeitet wurde

  bool ret_wert=false;

  // alle notwendigen Daten retten
  UDP_readUDPData();  
  // pruefen auf eigenen UDP-PORT
  if(RX_ETH_Data.udp_z_port==ETH_Setting.own_udp_port) {
    // wenn der Ziel-Port stimmt
    // pruefen auf Laenge der Daten
    if((RX_ETH_Data.udp_data_len>0) && (RX_ETH_Data.udp_data_len<UDP_DATA_LEN)) {
      // wenn Daten vorhanden sind
      // TRUE zurueckgeben
      ret_wert=true;
    }
    else {
      // laenge ist falsch
      RX_ETH_Data.udp_data_len=0;
    }
  }
  else {
    // wenn Ziel-Port falsch
    RX_ETH_Data.udp_data_len=0;
  }

  return(ret_wert);
}

//-----------------------------------------------------------------------------
void UDP_readUDPData(void) {
  // alle notwendigen Daten
  // vom UDP-Protokoll speichern

  unsigned char lo,hi;
  unsigned int uint;

  // Quell-Port speichern
  hi=RX_ETH_Data.raw_data[UDP_POS_Q_PORT+0];
  lo=RX_ETH_Data.raw_data[UDP_POS_Q_PORT+1];
  uint=(hi<<8);
  uint=(uint|lo);
  RX_ETH_Data.udp_q_port=uint;

  // Ziel-Port speichern
  hi=RX_ETH_Data.raw_data[UDP_POS_Z_PORT+0];
  lo=RX_ETH_Data.raw_data[UDP_POS_Z_PORT+1];
  uint=(hi<<8);
  uint=(uint|lo);
  RX_ETH_Data.udp_z_port=uint;

  // Laenge speichern
  hi=RX_ETH_Data.raw_data[UDP_POS_LEN+0];
  lo=RX_ETH_Data.raw_data[UDP_POS_LEN+1];
  uint=(hi<<8);
  uint=(uint|lo);
  // UDP-Header laenge abziehen
  uint=uint-UDP_HEADER_LEN;
  RX_ETH_Data.udp_data_len=uint;

}

//-----------------------------------------------------------------------------
void UDP_makeUDPPacket(void) {
  // UDP-Packet erstellen
  // Laenge der UDP-Daten muss in "TX_ETH_Data.udp_data_len" stehen
  // die Daten muessen ab Adr[42] stehen

  unsigned char lo,hi;
  unsigned int uint;

  //---------------------------
  // ETH-Header
  //---------------------------
  // Ziel MAC-Adresse
  RX_ETH_Data.raw_data[0]=HOST_ETH_Data.host_mac[0];
  RX_ETH_Data.raw_data[1]=HOST_ETH_Data.host_mac[1];
  RX_ETH_Data.raw_data[2]=HOST_ETH_Data.host_mac[2];
  RX_ETH_Data.raw_data[3]=HOST_ETH_Data.host_mac[3];
  RX_ETH_Data.raw_data[4]=HOST_ETH_Data.host_mac[4];
  RX_ETH_Data.raw_data[5]=HOST_ETH_Data.host_mac[5];

  // Quell MAC-Adresse
  RX_ETH_Data.raw_data[6]=ETH_Setting.own_mac[0];
  RX_ETH_Data.raw_data[7]=ETH_Setting.own_mac[1];
  RX_ETH_Data.raw_data[8]=ETH_Setting.own_mac[2];
  RX_ETH_Data.raw_data[9]=ETH_Setting.own_mac[3];
  RX_ETH_Data.raw_data[10]=ETH_Setting.own_mac[4];
  RX_ETH_Data.raw_data[11]=ETH_Setting.own_mac[5];

  // Typfield (IP-Packet = 0x0800)
  RX_ETH_Data.raw_data[12]=0x08;
  RX_ETH_Data.raw_data[13]=0x00;

  //---------------------------
  // IP-Header
  //---------------------------

  // IP-Version und Header Laenge (IP=4, Laenge=5)
  RX_ETH_Data.raw_data[14]=0x45;

  // Typ of Service
  RX_ETH_Data.raw_data[15]=0x00;

  // Laenge des IP-Frames (IP-Header + IP-Data) 
  uint=TX_ETH_Data.udp_data_len+UDP_FRAME_IU_LEN;
  lo=(uint&0x00FF);
  hi=(uint>>8);
  RX_ETH_Data.raw_data[16]=hi;
  RX_ETH_Data.raw_data[17]=lo;

  // eindeutige ID-Nummer vom IP-Protokoll
  TX_ETH_Data.ip_id_nr++;
  lo=(TX_ETH_Data.ip_id_nr&0x00FF);
  hi=(TX_ETH_Data.ip_id_nr>>8);
  RX_ETH_Data.raw_data[18]=hi;
  RX_ETH_Data.raw_data[19]=lo;

  // Offset (bei Protokoll Fragment)
  RX_ETH_Data.raw_data[20]=0x00;
  RX_ETH_Data.raw_data[21]=0x00;

  // Time To Live
  RX_ETH_Data.raw_data[22]=0x80;

  // IP-Protokoll-Typ (UDP = 0x11)
  RX_ETH_Data.raw_data[23]=0x11;

  // 24+25 = IP-Header Checksumme
  // wird spter von IP.h hinzugefuegt
  RX_ETH_Data.raw_data[24]=0x00;
  RX_ETH_Data.raw_data[25]=0x00;  

  // Quell IP-Adresse
  RX_ETH_Data.raw_data[26]=ETH_Setting.own_ip[0];
  RX_ETH_Data.raw_data[27]=ETH_Setting.own_ip[1];
  RX_ETH_Data.raw_data[28]=ETH_Setting.own_ip[2];
  RX_ETH_Data.raw_data[29]=ETH_Setting.own_ip[3];

  // Ziel IP-Adresse
  RX_ETH_Data.raw_data[30]=HOST_ETH_Data.host_ip[0];
  RX_ETH_Data.raw_data[31]=HOST_ETH_Data.host_ip[1];
  RX_ETH_Data.raw_data[32]=HOST_ETH_Data.host_ip[2];
  RX_ETH_Data.raw_data[33]=HOST_ETH_Data.host_ip[3];

  //---------------------------
  // UDP-Header
  //---------------------------
  // UDP-Quell-Port
  lo=(ETH_Setting.own_udp_port&0x00FF);
  hi=(ETH_Setting.own_udp_port>>8);
  RX_ETH_Data.raw_data[34]=hi;
  RX_ETH_Data.raw_data[35]=lo;

  // UDP-Ziel-Port
  lo=(ETH_Setting.own_udp_port&0x00FF);
  hi=(ETH_Setting.own_udp_port>>8);
  RX_ETH_Data.raw_data[36]=hi;
  RX_ETH_Data.raw_data[37]=lo;

  // laenge vom UDP-Frame
  // UDP-Header + UDP-Daten
  uint=TX_ETH_Data.udp_data_len+UDP_HEADER_LEN;
  lo=(uint&0x00FF);
  hi=(uint>>8);
  RX_ETH_Data.raw_data[38]=hi;
  RX_ETH_Data.raw_data[39]=lo;

  // 40+41 = UDP Checksumme
  // wird spter von UDP.h hinzugefuegt
  RX_ETH_Data.raw_data[40]=0x00;
  RX_ETH_Data.raw_data[41]=0x00;

  //---------------------------
  // UDP-Daten
  //---------------------------

  // Das Daten-Feld Adr 42 bis xx 
  // muss schon gefuellt sein

  // laenge festlegen
  TX_ETH_Data.eth_tx_len=TX_ETH_Data.udp_data_len+UDP_FRAME_EIU_LEN;

  //-----------------------------------------
  // die ETH-CRC wird vom EN28J60 angehaengt
  //-----------------------------------------

}

//-----------------------------------------------------------------------------
bool UDP_receivePacket(char *ptr) {
  // zum auslesen vom udp_string
  // rueckgabe von true, wenn etwas im puffer steht

  bool ret_wert=false;
  unsigned int n;

  if(RX_ETH_Data.udp_data_len>0) {
    // wenn Daten vorhanden sind

    // daten kopieren
    for(n=0;n<RX_ETH_Data.udp_data_len;n++) {
      ptr[n]=RX_ETH_Data.raw_data[UDP_POS_DATA+n];
    }
    ptr[n]=0x00;  // Stringendekennung

    // laenge wieder auf 0 setzen
    RX_ETH_Data.udp_data_len=0;
    ret_wert=true;
  }

  return(ret_wert);
}

//-----------------------------------------------------------------------------
void UDP_sendPacket(const char *ptr) {
  // zum senden vom udp_string

  unsigned int n;
  char c;

  // zuerst laenge auf 0
  TX_ETH_Data.udp_data_len=0;  

  // daten kopieren
  for(n=0;n<UDP_DATA_LEN;n++) {
    c=ptr[n];
    if(c==0) {
      // wenn endekennung erreicht
      TX_ETH_Data.udp_data_len=n;
      break;
    } 
    else {
      RX_ETH_Data.raw_data[UDP_POS_DATA+n]=c;
    }   
  }
  
  if(TX_ETH_Data.udp_data_len>0) {
    // udp-packet erstellen
    UDP_makeUDPPacket();
    // UDP-Checksumme hinzufuegen
    UDP_makeUDPChecksum();
  }  
}


//-----------------------------------------------------------------------------
void UDP_makeUDPChecksum(void) {
  // fuegt die UDP-Checksumme in den Frame ein

  // TODO : muss noch programmiert werden    
}

#endif  // Ende von #define UB_ETH_UDP


